﻿using System.Security.Cryptography.X509Certificates;
using SoapEndpoint.Service.Proxy.Models.Systeem;

namespace SoapEndpoint.Tester.Infrastructure;

public static class CertificateHelper
{
    public static X509Certificate2 FindCertificateByThumbprint(string thumbprint)
    {
        using var store = new X509Store(StoreName.My, StoreLocation.CurrentUser, OpenFlags.ReadOnly);
        var certificates = store.Certificates.Find(X509FindType.FindByThumbprint, thumbprint, true);
        return certificates.First();
    }

    public static OrganisatieType? GetOrganisationType(CertificateItem certificate)
    {
        if (certificate.Certificate.Subject.Contains("OU=SHV"))
        {
            return OrganisatieType.Schuldhulpverlener;
        }

        if (certificate.Certificate.Subject.Contains("OU=SE"))
        {
            return OrganisatieType.Schuldeiser;
        }

        if (certificate.Name.Contains("SHV"))
        {
            return OrganisatieType.Schuldhulpverlener;
        }

        if (certificate.Name.Contains("SE"))
        {
            return OrganisatieType.Schuldeiser;
        }

        return default;
    }


    public static IEnumerable<CertificateItem> FindCertificates(X509Certificate2 rootCertificate = null)
    {
        var result = new List<CertificateItem>();
        using var store = new X509Store(StoreName.My, StoreLocation.CurrentUser, OpenFlags.ReadOnly);

        foreach (var certificate in store.Certificates)
        {
            if (rootCertificate != null && certificate.Issuer != rootCertificate.Issuer)
            {
                continue;
            }

            result.Add(new CertificateItem(certificate));
        }

        return result;
    }
}