using SoapEndpoint.Service.Flows.Bewindvoering.v1.LogbookService;
using SoapEndpoint.Service.Proxy.Models.Systeem;
using SoapEndpoint.Service.Proxy.ServiceProxies.Systeem.v2.ErrorMessage;
using SoapEndpoint.Tester.Scenario.Flow.Bewindvoering;

namespace SoapEndpoint.Tester.Scenario.Implementations.Bewindvoering;

public class SeSectorNutsScenario : BewindvoeringScenarioBase
{
    public override string Name => "SE sector nuts scenario";

    public override AdresboekOrganisatieModel SeOrganization =>
        Manager.Organizations.SingleOrDefault(p => p.Referentienummer.ToString()
            .Equals(Program.Configuration["SeUtilityCompanyTestOrganizationReference"],
                StringComparison.CurrentCultureIgnoreCase));

    protected override async Task ExecuteAsync()
    {
        StartNewScenario("Start bewind");

        await Manager.SHV_FlowAsync(Conversation, null,
            new ShvFlowItem(BerichtType.SHV_StartBewind, BerichtType.SE_AntwoordMeldingBewind, true));

        StartNewScenario("Actualisatie bewindgegevens");

        await Manager.SHV_FlowAsync(Conversation, null,
            new ShvFlowItem(BerichtType.SHV_ActualisatieBewindgegevens, BerichtType.SE_AntwoordMeldingBewind, true));

        StartNewScenario("Eind bewind");

        await Manager.SHV_FlowAsync(Conversation, null,
            new ShvFlowItem(BerichtType.SHV_EindeBewind, BerichtType.SE_AntwoordMeldingBewind, true));

        await SendInvalidSectorTypeMesages();
    }

    private async Task SendInvalidSectorTypeMesages()
    {
        var messageTypes = new List<BerichtType>
        {
            BerichtType.SHV_StartBewindPubliek,
            BerichtType.SHV_ActualisatieBewindgegevensPubliek,
            BerichtType.SHV_EindeBewindPubliek
        };

        foreach (var messageType in messageTypes)
        {
            StartNewScenario($"Foutsituatie: {messageType}");

            await Manager.SHV_SendIncorrectMessageAsync(Conversation, messageType, null,
                ShvKnooppuntFoutcode.Incorrectberichttypevoorsectorvanontvanger);
        }
    }
}