﻿using System.Reflection;
using Infrastructure.Utils.Xml;
using SoapEndpoint.Service.Proxy.Models.Flows;
using SoapEndpoint.Service.Proxy.Models.Systeem;
using SoapEndpoint.Service.Proxy.ResourceItems;

namespace SoapEndpoint.Service.Proxy.Infrastructure;

public static class MessageHelper
{
    public static async Task<ShvKnooppuntBerichtBevestigingModel> ExecActionAsync<TMessageType>(
        bool validateXml,
        string content,
        BerichtHeaderModel header,
        ResourceItem<TMessageType> item,
        Func<Task<ShvKnooppuntBerichtBevestigingModel>> proxyCallAsync,
        Action<string, bool> logAction, string schemaVersion = "1.0") where TMessageType : Enum
    {
        var result = default(ShvKnooppuntBerichtBevestigingModel);
        try
        {
            if (validateXml && !ValidateXml(header, logAction, content, item.FlowType, schemaVersion))
            {
                return ShvKnooppuntBerichtBevestigingModel.ErrorResult;
            }

            result = await proxyCallAsync();

            if (result.Success)
            {
                logAction($"{item.Name} sent", true);
            }
            else
            {
                logAction($"Error sending:{item.Name}. Errors:", false);
                foreach (var fout in result.Errors) logAction(fout, true);
            }
        }
        catch (Exception ex)
        {
            logAction(ex.ToString(), false);
        }

        return result ?? ShvKnooppuntBerichtBevestigingModel.ErrorResult;
    }

    private static bool ValidateXml(BerichtHeaderModel header, Action<string, bool> logAction,
        string content, FlowType flowType, string schemaVersion)
    {
        var validator = new XmlValidator();
        var headerContent = DataContractSerializeHelper.Serialize(header);

        // Validate Xsd 
        logAction("Validating message header...", true);

        var headerValidationResult = validator.ValidateDocument(headerContent, Assembly.GetExecutingAssembly());
        if (!ProcessValidationResult(headerValidationResult, logAction))
        {
            return false;
        }

        logAction("Validating message content...", true);

        var validationResult = validator.ValidateDocument(content, Assembly.GetExecutingAssembly(),
            new FlowXmlValidationOptions { FlowType = flowType.ToString(), SchemaVersion = schemaVersion });

        return ProcessValidationResult(validationResult, logAction);
    }

    private static bool ProcessValidationResult(XmlValidationResult result, Action<string, bool> logAction)
    {
        if (!result.Success)
        {
            logAction($"XSD validation errors ({result.Errors.Count}): ", result.Success);
            foreach (var fout in result.Errors) logAction(fout, result.Success);

            return result.Success;
        }

        logAction("No XSD validation errors", result.Success);
        return result.Success;
    }
}