﻿using System.Security.Cryptography.X509Certificates;
using AutoMapper;
using SoapEndpoint.Service.Flows.SbEnSk.v1.LogbookService;
using SoapEndpoint.Service.Flows.SbEnSk.v1.SeService;
using SoapEndpoint.Service.Flows.SbEnSk.v1.ShvService;
using SoapEndpoint.Service.Proxy.Models.Flows;
using SoapEndpoint.Service.Proxy.ServiceProxies.Flows;
using SoapEndpoint.Service.Proxy.ServiceProxies.Flows.SbEnSk.v1;
using SoapEndpoint.Service.Proxy.ServiceProxies.Systeem;
using SoapEndpoint.Service.Proxy.ServiceProxies.Systeem.v1;
using SoapEndpoint.Service.Systeem.v1.AddressService;
using SoapEndpoint.Service.Systeem.v1.ConnectionTestService;
using SoapEndpoint.Service.Systeem.v1.LogbookService;
using SoapEndpoint.Service.Systeem.v1.MessageBoxService;
using static SoapEndpoint.Service.Proxy.Infrastructure.ServiceClientHelper;
using LogbookProxy = SoapEndpoint.Service.Proxy.ServiceProxies.Systeem.v1.LogbookProxy;

namespace SoapEndpoint.Service.Proxy.Infrastructure;

public class ServiceProxyResolver
{
    private readonly IMapper _mapper;

    public ServiceProxyResolver(IMapper mapper)
    {
        _mapper = mapper;
    }

    public IMessageProxy<TMessageType> GetShvProxy<TMessageType>(X509Certificate2 certificate,
        FlowType flowType, Version flowVersion, string? url = null) where TMessageType : Enum
    {
        switch (flowType)
        {
            case FlowType.SbEnSk:

                if (flowVersion.Major == SystemVersions.V1_0.Major)
                {
                    var client = ConfigureClient(new SoapShvServiceClient(),
                        certificate, url);
                    return (new ShvProxy(_mapper, client) as IMessageProxy<TMessageType>)!;
                }

                throw new InvalidOperationException(
                    $"No SHV proxy available for the selected flow type: {flowType} and Version: {flowVersion}");

            case FlowType.Schuldbemiddeling:

                if (flowVersion.Major == SystemVersions.V1_0.Major)
                {
                    var client = ConfigureClient(
                        new Flows.Schuldbemiddeling.v1.ShvService.SoapShvServiceClient(),
                        certificate, url);
                    return (new ServiceProxies.Flows.Schuldbemiddeling.v1.ShvProxy(_mapper, client) as
                        IMessageProxy<TMessageType>)!;
                }

                throw new InvalidOperationException(
                    $"No SHV proxy available for the selected flow type: {flowType} and Version: {flowVersion}");

            case FlowType.Waarborgfonds:
                if (flowVersion.Major == SystemVersions.V1_0.Major)
                {
                    var client = ConfigureClient(
                        new Flows.Waarborgfonds.v1.ShvService.SoapShvServiceClient(),
                        certificate, url);
                    return (new ServiceProxies.Flows.Waarborgfonds.v1.ShvProxy(_mapper, client) as
                        IMessageProxy<TMessageType>)!;
                }

                throw new InvalidOperationException(
                    $"No SHV proxy available for the selected flow type: {flowType} and Version: {flowVersion}");

            case FlowType.Inventarisatie:
                if (flowVersion.Major == SystemVersions.V1_0.Major)
                {
                    var client = ConfigureClient(
                        new Flows.Inventarisatie.v1.ShvService.SoapShvServiceClient(),
                        certificate, url);
                    return (new ServiceProxies.Flows.Inventarisatie.v1.ShvProxy(_mapper, client) as
                        IMessageProxy<TMessageType>)!;
                }

                throw new InvalidOperationException(
                    $"No SHV proxy available for the selected flow type: {flowType} and Version: {flowVersion}");

            case FlowType.BrEnHf:
                if (flowVersion.Major == SystemVersions.V1_0.Major)
                {
                    var client = ConfigureClient(
                        new Flows.BrEnHf.v1.ShvService.SoapShvServiceClient(),
                        certificate, url);
                    return (new ServiceProxies.Flows.BrEnHf.v1.ShvProxy(_mapper, client) as
                        IMessageProxy<TMessageType>)!;
                }

                throw new InvalidOperationException(
                    $"No SHV proxy available for the selected flow type: {flowType} and Version: {flowVersion}");

            case FlowType.Bewindvoering:
                if (flowVersion.Major == SystemVersions.V1_0.Major)
                {
                    var client = ConfigureClient(
                        new Flows.Bewindvoering.v1.ShvService.SoapShvServiceClient(),
                        certificate, url);
                    return (new ServiceProxies.Flows.Bewindvoering.v1.ShvProxy(_mapper, client) as
                        IMessageProxy<TMessageType>)!;
                }

                throw new InvalidOperationException(
                    $"No SHV proxy available for the selected flow type: {flowType} and Version: {flowVersion}");

            case FlowType.PauzeSignaal3:
                if (flowVersion.Major == SystemVersions.V1_0.Major)
                {
                    var client = ConfigureClient(
                        new Flows.PauzeSignaal3.v1.ShvService.SoapShvServiceClient(),
                        certificate, url);
                    return (new ServiceProxies.Flows.PauzeSignaal3.v1.ShvProxy(_mapper, client) as
                        IMessageProxy<TMessageType>)!;
                }

                throw new InvalidOperationException(
                    $"No SHV proxy available for the selected flow type: {flowType} and Version: {flowVersion}");

            case FlowType.ShvSignaal:
                if (flowVersion.Major == SystemVersions.V1_0.Major)
                {
                    var client = ConfigureClient(
                        new Flows.ShvSignaal.v1.ShvService.SoapShvServiceClient(),
                        certificate, url);
                    return (new ServiceProxies.Flows.ShvSignaal.v1.ShvProxy(_mapper, client) as
                        IMessageProxy<TMessageType>)!;
                }

                throw new InvalidOperationException(
                    $"No SHV proxy available for the selected flow type: {flowType} and Version: {flowVersion}");

            default: throw new InvalidOperationException($"Specified flow type {flowType} was not found");
        }
    }

    public IMessageProxy<TMessageType> GetSeProxy<TMessageType>(X509Certificate2 certificate,
        FlowType flowType, Version flowVersion, string? url = null) where TMessageType : Enum
    {
        switch (flowType)
        {
            case FlowType.SbEnSk:

                if (flowVersion.Major == SystemVersions.V1_0.Major)
                {
                    var client = ConfigureClient(new SoapSeServiceClient(), certificate,
                        url);
                    return (new SeProxy(_mapper, client) as IMessageProxy<TMessageType>)!;
                }

                throw new InvalidOperationException(
                    $"No SE proxy available for the selected flow type: {flowType} and Version: {flowVersion}");

            case FlowType.Waarborgfonds:

                if (flowVersion.Major == SystemVersions.V1_0.Major)
                {
                    var client = ConfigureClient(
                        new Flows.Waarborgfonds.v1.SeService.SoapSeServiceClient(), certificate,
                        url);
                    return (new ServiceProxies.Flows.Waarborgfonds.v1.SeProxy(_mapper, client) as
                        IMessageProxy<TMessageType>)!;
                }

                throw new InvalidOperationException(
                    $"No SE proxy available for the selected flow type: {flowType} and Version: {flowVersion}");

            case FlowType.Inventarisatie:

                if (flowVersion.Major == SystemVersions.V1_0.Major)
                {
                    var client = ConfigureClient(
                        new Flows.Inventarisatie.v1.SeService.SoapSeServiceClient(), certificate,
                        url);
                    return (new ServiceProxies.Flows.Inventarisatie.v1.SeProxy(_mapper, client) as
                        IMessageProxy<TMessageType>)!;
                }

                throw new InvalidOperationException(
                    $"No SE proxy available for the selected flow type: {flowType} and Version: {flowVersion}");

            case FlowType.BrEnHf:

                if (flowVersion.Major == SystemVersions.V1_0.Major)
                {
                    var client = ConfigureClient(new Flows.BrEnHf.v1.SeService.SoapSeServiceClient(),
                        certificate,
                        url);
                    return (new ServiceProxies.Flows.BrEnHf.v1.SeProxy(_mapper, client) as
                        IMessageProxy<TMessageType>)!;
                }

                throw new InvalidOperationException(
                    $"No SE proxy available for the selected flow type: {flowType} and Version: {flowVersion}");

            case FlowType.Bewindvoering:

                if (flowVersion.Major == SystemVersions.V1_0.Major)
                {
                    var client = ConfigureClient(new Flows.Bewindvoering.v1.SeService.SoapSeServiceClient(),
                        certificate,
                        url);
                    return (new ServiceProxies.Flows.Bewindvoering.v1.SeProxy(_mapper, client) as
                        IMessageProxy<TMessageType>)!;
                }

                throw new InvalidOperationException(
                    $"No SE proxy available for the selected flow type: {flowType} and Version: {flowVersion}");

            case FlowType.PauzeSignaal3:

                if (flowVersion.Major == SystemVersions.V1_0.Major)
                {
                    var client = ConfigureClient(new Flows.PauzeSignaal3.v1.SeService.SoapSeServiceClient(),
                        certificate,
                        url);
                    return (new ServiceProxies.Flows.PauzeSignaal3.v1.SeProxy(_mapper, client) as
                        IMessageProxy<TMessageType>)!;
                }

                throw new InvalidOperationException(
                    $"No SE proxy available for the selected flow type: {flowType} and Version: {flowVersion}");

            case FlowType.Schuldbemiddeling:

                if (flowVersion.Major == SystemVersions.V1_0.Major)
                {
                    var client = ConfigureClient(
                        new Flows.Schuldbemiddeling.v1.SeService.SoapSeServiceClient(),
                        certificate, url);
                    return (new ServiceProxies.Flows.Schuldbemiddeling.v1.SeProxy(_mapper,
                        client) as IMessageProxy<TMessageType>)!;
                }

                throw new InvalidOperationException(
                    $"No SE proxy available for the selected flow type: {flowType} and Version: {flowVersion}");

            default: throw new InvalidOperationException($"Specified flow type {flowType} was not found");
        }
    }

    public IMessageBoxProxy GetMessageBoxProxy(X509Certificate2 certificate, Version version, string? url = null)
    {
        if (version.Major == SystemVersions.V1_0.Major)
        {
            var client = ConfigureClient(new MessageBoxServiceClient(),
                certificate, url);
            return new MessageBoxProxy(_mapper, client);
        }

        if (version.Major == SystemVersions.V2_0.Major)
        {
            var client = ConfigureClient(new Systeem.v2.MessageBoxService.MessageBoxServiceClient(),
                certificate, url);
            return new ServiceProxies.Systeem.v2.MessageBoxProxy(_mapper, client);
        }

        throw new NotSupportedException();
    }

    public IAddressBookProxy GetAddressProxy(X509Certificate2 certificate, Version version, string? url = null)
    {
        if (version.Major == SystemVersions.V1_0.Major)
        {
            var client = ConfigureClient(new AddressServiceClient(), certificate,
                url);
            return new AddressBookProxy(_mapper, client);
        }

        if (version.Major == SystemVersions.V2_0.Major)
        {
            var client = ConfigureClient(new Systeem.v2.AddressService.AddressServiceClient(), certificate,
                url);
            return new ServiceProxies.Systeem.v2.AddressBookProxy(_mapper, client);
        }

        throw new NotSupportedException();
    }

    public ILogbookProxy<TMessageType> GetLogbookProxy<TMessageType>(X509Certificate2 certificate,
        Version version, FlowType flowType, string? url = null) where TMessageType : Enum
    {
        if (version.Major == SystemVersions.V1_0.Major)
        {
            return (new LogbookProxy(_mapper, ConfigureClient(new LogbookServiceClient(),
                certificate,
                url)) as ILogbookProxy<TMessageType>)!;
        }

        if (version.Major == SystemVersions.V2_0.Major)
        {
            switch (flowType)
            {
                case FlowType.Schuldbemiddeling:
                    return (new LogbookProxy(_mapper,
                        ConfigureClient(new LogbookServiceClient(), certificate,
                            url)) as ILogbookProxy<TMessageType>)!;
                case FlowType.SbEnSk:
                    return (new ServiceProxies.Flows.SbEnSk.v1.LogbookProxy(_mapper,
                        ConfigureClient(
                            new
                                SoapLogbookServiceOf_LogboekBericht_LogboekZoekbericht_ConversatieLogboekBericht_ConversatieLogboekZoekberichtClient(),
                            certificate, url)) as ILogbookProxy<TMessageType>)!;
                case FlowType.Waarborgfonds:
                    return (new ServiceProxies.Flows.Waarborgfonds.v1.LogbookProxy(_mapper,
                        ConfigureClient(
                            new Flows.Waarborgfonds.v1.LogbookService.SoapLogbookServiceOf_LogboekBericht_LogboekZoekbericht_ConversatieLogboekBericht_ConversatieLogboekZoekberichtClient(),
                            certificate, url)) as ILogbookProxy<TMessageType>)!;
                case FlowType.Inventarisatie:
                    return (new ServiceProxies.Flows.Inventarisatie.v1.LogbookProxy(_mapper,
                        ConfigureClient(
                            new Flows.Inventarisatie.v1.LogbookService.SoapLogbookServiceOf_LogboekBericht_LogboekZoekbericht_ConversatieLogboekBericht_ConversatieLogboekZoekberichtClient(),
                            certificate, url)) as ILogbookProxy<TMessageType>)!;
                case FlowType.BrEnHf:
                    return (new ServiceProxies.Flows.BrEnHf.v1.LogbookProxy(_mapper,
                        ConfigureClient(
                            new Flows.BrEnHf.v1.LogbookService.SoapLogbookServiceOf_LogboekBericht_LogboekZoekbericht_ConversatieLogboekBericht_ConversatieLogboekZoekberichtClient(),
                            certificate, url)) as ILogbookProxy<TMessageType>)!;
                case FlowType.Bewindvoering:
                    return (new ServiceProxies.Flows.Bewindvoering.v1.LogbookProxy(_mapper,
                        ConfigureClient(
                            new Flows.Bewindvoering.v1.LogbookService.SoapLogbookServiceOf_LogboekBericht_LogboekZoekbericht_ConversatieLogboekBericht_ConversatieLogboekZoekberichtClient(),
                            certificate, url)) as ILogbookProxy<TMessageType>)!;
                case FlowType.PauzeSignaal3:
                    return (new ServiceProxies.Flows.PauzeSignaal3.v1.LogbookProxy(_mapper,
                        ConfigureClient(
                            new Flows.PauzeSignaal3.v1.LogbookService.SoapLogbookServiceOf_LogboekBericht_LogboekZoekbericht_ConversatieLogboekBericht_ConversatieLogboekZoekberichtClient(),
                            certificate, url)) as ILogbookProxy<TMessageType>)!;
                case FlowType.ShvSignaal:
                    return (new ServiceProxies.Flows.ShvSignaal.v1.LogbookProxy(_mapper,
                        ConfigureClient(
                            new Flows.ShvSignaal.v1.LogbookService.SoapLogbookServiceOf_LogboekBericht_LogboekZoekbericht_ConversatieLogboekBericht_ConversatieLogboekZoekberichtClient(),
                            certificate, url)) as ILogbookProxy<TMessageType>)!;
                default:
                    throw new InvalidOperationException(
                        $"No SHV proxy available for the selected flow type: {flowType}");
            }
        }

        throw new NotSupportedException();
    }

    public IConnectionTestProxy GetConnectionTestProxy(X509Certificate2 certificate, Version version,
        string? url = null)
    {
        if (version.Major == SystemVersions.V1_0.Major)
        {
            var client = ConfigureClient(new ConnectionTestServiceClient(),
                certificate, url);
            return new ConnectionTestProxy(client);
        }

        if (version.Major == SystemVersions.V2_0.Major)
        {
            var client = ConfigureClient(new Systeem.v2.ConnectionTestService.ConnectionTestServiceClient(),
                certificate, url);
            return new ServiceProxies.Systeem.v2.ConnectionTestProxy(client);
        }

        throw new NotSupportedException();
    }
}